#Supplemental Data (Model Code)
#Manuscript: "Integrated modeling of phylogenies, species traits, and environmental gradients to better predict biogeographic distributions"

#All analysis and modeling were performed using the open-source R language for
#statistical computing (R Core Team, 2017).
#Copies of the R code and data to fit the PhyCoRT model presented in
#this study will be made available on a publicly accessible data repository on
#article acceptance.

#The code provided for review below will construct and execute the three-tiered joint PhyCoRT model described in the 
#manuscript methods section, as well as, the Model0, Model1, and NSMod models discussed for comparison.

#Included with this script were three comma seperated value text files and one RData file that must be in the R Working Directory:
###File 1: Titled "vcv.csv" is the phylogentic variance-covariance matrix described in the manscript.
###File 2: Titled "morpho.csv" inlcludes the morphological data for all modeled taxa.
###File 3: Titled "species.csv" includes the georeferenced species occurrences (Phyllotis amicus) with associated environmental covariates.
###File 4: Titled "mesh.RData" inlcludes the triangulated mesh that defines study domain boundaries.


# MODEL CODE BELOW ############################################################################################

#Required Libraries
#The INLA package is not avialable on CRAN, but, can be installed using the code below or 
#downloaded from the INLA website (http://www.r-inla.org)
#install.packages("INLA", repos=c(getOption("repos"), INLA="https://inla.r-inla-download.org/R/stable"), dep=TRUE)
library(INLA)
library(rgl)
library(dplyr)

#Load Data
FE.df = read.csv("species.csv") #Species and covariate data (column names described further below)
FE2.df = read.csv("morpho.csv") #Species morphology (column names described further below)

#load and view mesh
load("mesh.RData")
plot(mesh.dom, rgl = TRUE, draw.edges = TRUE, #Will open in new window
     draw.segments = TRUE, draw.vertices =FALSE)

#Covariance matrix
VCV.mat = read.csv("vcv.csv") #Phylogentic variance-covariance matrix
colnames(VCV.mat) = 1:12
rownames(VCV.mat) = 1:12
VCV.mat = as.matrix(VCV.mat)


#Initial specification for spatial prior
#Will be updated later
spde1 = inla.spde2.pcmatern(mesh.dom, alpha = 2,
                            prior.range=c(0.5, 0.9),  
                            prior.sigma=c(1, 0.5), 
                            constr = TRUE)

#Spatial Indices
#the suffix "x" indicates that index will be used as a shared spatial field
field1 = inla.spde.make.index("field1", #Index for tier 1 morphometrics (PC3)
                               spde1$n.spde)

field1x = inla.spde.make.index("field1x", #Copy of tier 1 index for tier 2
                               spde1$n.spde) 

field1xx = inla.spde.make.index("field1xx", #Copy of tier 1 index for tier 3
                               spde1$n.spde)

field2 = inla.spde.make.index("field2", #Index for tier 1 morphometrics (PC4)
                              spde1$n.spde) 

field2x = inla.spde.make.index("field2x", #Copy of tier 2 index for tier 3
                               spde1$n.spde)

field3 = inla.spde.make.index("field3", #Index for tier 3 (focal species occurrence)
                              spde1$n.spde)



#Relate morphological samples to mesh
locs = cbind(FE2.df$Long, FE2.df$Lat) #geographic coordinates

#convert to 3D Cartesian coordinates
locs = inla.mesh.map(locs, 
                     projection = "longlat", 
                     inverse = TRUE)
#projection matrix
Aphy = inla.spde.make.A(mesh.dom, loc=locs) #relate locations to mesh
Aphy2 = Aphy #copy of matrix


#Relate focal species occurrences to to mesh
locs = cbind(FE.df$Long, FE.df$Lat) #Coordinates

#convert to 3D Cartesian coordinates
locs = inla.mesh.map(locs, 
                     projection = "longlat", 
                     inverse = TRUE)
#projection matrix
A.osi = inla.spde.make.A(mesh.dom, loc=locs) #relate locations to mesh




#Organize data for model fitting
#Creatng tier specific data stacks

#Construct data stack for model tier 1
tier1.lst = list(c(field1, #spatial field
                 list(intercept1 = 1)), #intercept
               list(bio3 = FE2.df[,"bio3"], #Environmental variable 1 (Isothermality)
                    bio19 = FE2.df[,"bio19"],  #Environmental variable 2 (precipitation)
                    VcVx = FE2.df[,"VcVx"], #Index linking each taxa to corresponding location in vcv matrix
                    Sex = FE2.df[,"Sex2"], #specimen sex
                    Age = FE2.df[,"Age2"], #specimen age
                    Clust = FE2.df[,"Cluster_ID"])) #repeated measures effect (species*locations)

#Column "Morph3" is the 3rd PC from morphological analysis as described in manuscript (Tier 1 dependent variable)
Phy.stk = inla.stack(data = list(Y = cbind(FE2.df$Morph3, NA, NA)), #three elements, first holds response, second two reserved
                     A = list(Aphy, 1), #projection matrix
                     effects = tier1.lst,    #level-specific covariates and indicies
                     tag = "phy0") #arbitrary name as label




#Construct data stack for model tier 1
tier2.lst = list(c(field2, #spatial index for tier 2
                   field1x, #copy of spatial index from tier 1
                 list(intercept2 = 1)),
               list(bio2 = FE2.df[,"bio2"], #Environmental covariate (mean diurnal temperature)
                    VcVx = FE2.df[,"VcVx"], #Index to vcv matrix
                    Sex = FE2.df[,"Sex2"], #specimen sex
                    Age = FE2.df[,"Age2"], #specimen age
                    Clust = FE2.df[,"Cluster_ID"])) #repeated measures effect (species*locations)

#Column "Morph4" is the 4th PC from morphological analysis as described in manuscript (Tier 2 dependent variable)
Phy.stk2 = inla.stack(data = list(Y = cbind(NA, FE2.df$Morph4, NA)), #three elements, data in second position
                      A = list(Aphy2, 1), #projection matrix
                      effects = tier2.lst,  #level-specific covariates and indicies 
                      tag = "phy2") #arbitrary name as label



##Tier 3 data stack
##Data stack for focal species (Phyllotis amicus)
FE.lst = list(c(field3, #spatial field for tier 3
                field1xx, #copy of spatiale field from tier 1
                field2x,  #copy of spatiale field from tier 2
                list(intercept3 = 1)), #intercept
              list(bio1 = FE.df[,"bio1"], #Environmental covariate (annual mean temperature)
                   bio15 = FE.df[,"bio15"], #Environmental covariate (precipitation seasonality)
                   aPET = FE.df[,"aPET"], #Environmental covariate (evapotranspiration)
                   Pop = FE.df[,"Pop"])) #Population density

Focal.stk = inla.stack(data = list(Y = cbind(NA, NA, FE.df$OBS2)), #three elements, species presence/absence ("OBS2") in last position
                    A = list(A.osi, 1), #projection matrix
                    effects = FE.lst,  #covariate list 
                    tag = "osi.1j") #name as label


#Combine all three data stacks
J.Stack = inla.stack(Phy.stk, Phy.stk2, Focal.stk)





###Update PC Prior to include constraints
n.data = dim(FE.df)[1] #dimensions of data
n.covariates = 5

X = cbind(rep(1,n.data), #tier 3 covariates
          FE.df$bio1,
          FE.df$bio15,
          FE.df$aPET,
          FE.df$Pop)

Q = qr.Q(qr(X))

spde1 = inla.spde2.pcmatern(mesh.dom,
                            prior.range=c(0.5, 0.9),  
                            prior.sigma=c(1, 0.5),  
                            constr = TRUE,
                            extraconstr = list(A = as.matrix(t(Q)%*%A.osi), #focal species matrix
                                               e = rep(0, n.covariates)))  



###Priors and Formula for PhyCoRT Model
error.prec = list(hyper=list(theta=list(prior = "pc.prec", param=c(1, 0.01))))
MorphPC = list(theta=list(prior = "normal", param=c(0, 10))) 
MorphPC2 = list(theta=list(prior = "normal", param=c(1, 70))) 
MorphPC3 = list(theta=list(prior = "normal", param=c(0, 20)))


Frm.J = Y ~ -1 +  intercept1 + #intercept tier 1
                  intercept2 + #intercept tier 2
                  intercept3 + #intercept tier 3
                  f(field1, #spatial effect tier 1
                    model = spde1) +
                  f(field2, #spatial effect tier 2
                    model = spde1) +
                  f(field3, #spatial effect tier 3
                    model = spde1) +
                  f(field1x, #Shared field tier1:tier2
                    copy = "field1", 
                    fixed = FALSE) + 
                  f(field2x,  #Shared field tier2:tier3
                    copy = "field2", 
                    fixed = FALSE) + 
                  f(field1xx, #Shared field tier1:tier3
                    copy = "field1", 
                    fixed = FALSE) +
                  f(VcVx, #index to vcv matrix
                    model="generic0", 
                    Cmatrix = VcV, #Label for vcv matrix in "ModelJ" below.
                    hyper = MorphPC3,
                    rankdef=1, 
                    constr=TRUE, 
                    diagonal=1e-05) +
                  f(Sex, #specimen sex covariate
                    model="iid",
                    hyper = MorphPC) +
                  f(Age, #specimen age covariate
                    model="iid",
                    hyper = MorphPC) +
                  f(Clust, #Repeated measures
                    model="iid",
                    hyper = MorphPC2) +
                  bio1 + bio15 + aPET + Pop + bio2 + bio3 + bio19 #environmental covariates (differ by data stack)


###Execute Model

#thetaJ = ModelJ$internal.summary.hyperpar$mean (from prior model run, to speed up convergence)
thetaJ = c(-0.5084671, 0.1211920, 0.8615496, -4.3424533, 0.0897133, -0.6534892, 0.1898479, -2.7480555, 0.9827745, 0.2222411,
            0.1011618, 0.1252657, 1.4164456, 1.2065828, 0.9324327, -0.4407870)


ModelJ = inla(Frm.J, #Formula
              data = inla.stack.data(J.Stack, #combined data stack
                                     spde1=spde1, #PC Prior
                                     VcV = VCV.mat), #vcv matrix
              family = c("gaussian", "gaussian", "zeroinflatedbinomial1"), #Likelihoods by tier
              verbose = TRUE, #verbose results printed
              control.family = list(error.prec, error.prec, error.prec),#error precision
              control.fixed = list(prec = 0.1, prec.intercept = 0.001), #Intercept/fixed effects
              control.predictor = list(
                                    A = inla.stack.A(J.Stack), #combined data stack
                                    compute = TRUE, #return fitted values
                                    link = 1), #default log links
              control.mode = list(restart = TRUE, theta = thetaJ), #use theta from prior run
              control.inla = list(int.strategy = "eb"), #Empirical Bayes
              control.results = list(return.marginals.random = TRUE, #return marginals
                                     return.marginals.predictor = TRUE),
              control.compute=list(dic = TRUE, cpo = TRUE, waic = TRUE)) #calculate comparative metrics


summary(ModelJ)

##Model Metrics
#DIC (tier 3)
tapply(ModelJ$dic$local.dic, ModelJ$dic$family, sum)[3]

#WAIC (Tier 3)
tapply(ModelJ$waic$local.waic, ModelJ$dic$family, sum)[3]






##############################################################################################################
################  Comparison Models  #########################################################################
##############################################################################################################

#Model0, Model1, and NSMod from manuscript.

#Organize data as data stack
#Same covariates as the PhyCoRT Tier 3 Model, but, dropping shared spatial fields.   
FE.Ind.lst = list(c(field3, #spatial field for tier 3
                  list(intercept3 = 1)), #intercept
                list(bio1 = FE.df[,"bio1"], #Environmental covariate (annual mean temperature)
                     bio15 = FE.df[,"bio15"], #Environmental covariate (precipitation seasonality)
                     aPET = FE.df[,"aPET"], #Environmental covariate (evapotranspiration)
                     Pop = FE.df[,"Pop"])) #Population density

Ind.stk = inla.stack(data = list(Y = FE.df$OBS2), #Single response indicating species presence/absence ("OBS2")
                       A = list(A.osi, 1), #projection matrix for observations
                       effects = FE.Ind.lst,  #covariate list 
                       tag = "Ind.mod") #name as label

#####################################
#### Model0 (Spatial effect only) ###
#####################################

Frm0 = Y ~ -1 + intercept3 + #intercept
                 f(field3, #spatial effect only
                   model=spde) 

#theta0 = Model0$internal.summary.hyperpar$mean
theta0 = c(0.793335, -1.710971, 1.481912) #previous run

Model0 = inla(Frm0, 
              data = inla.stack.data(Ind.stk, #Data stack
                                     spde=spde1), 
              family = "zeroinflatedbinomial1", 
              verbose = TRUE,
              control.family = list(error.prec),
              control.fixed = list(prec = 0.1, prec.intercept = 0.001), 
              control.predictor = list(
                A = inla.stack.A(Ind.stk), 
                compute = TRUE, 
                link = 1), 
              control.mode = list(restart = TRUE, theta = theta0),
              control.results = list(return.marginals.random = TRUE,
                                     return.marginals.predictor = TRUE),
              control.compute=list(dic = TRUE, cpo = TRUE, waic = TRUE)) 

summary(Model0)



#####################################
#### Model1 (Base spatial model) ###
#####################################

Frm1 = Y ~ -1 + intercept3 + 
                  f(field3, model=spde) + 
                  bio1 + bio15 + aPET + Pop


#theta1 = Model1$internal.summary.hyperpar$mean
theta1 = c(-0.2018038, -0.9674923, 1.3514013) 


Model1 = inla(Frm1, 
              data = inla.stack.data(Ind.stk, 
                                     spde=spde1), 
              family = "zeroinflatedbinomial1", 
              verbose = TRUE,
              control.family = list(error.prec),
              control.fixed = list(prec = 0.1, prec.intercept = 0.0001), 
              control.predictor = list(
                A = inla.stack.A(Ind.stk), 
                compute = TRUE, 
                link = 1), 
              control.mode = list(restart = TRUE, theta = theta1),
              control.results = list(return.marginals.random = TRUE,
                                     return.marginals.predictor = TRUE),
              control.compute=list(dic = TRUE, cpo = TRUE, waic = TRUE)) 

summary(Model1)




#####################################
#### NSMod (Non Spatial Model) ###
#####################################

Frm.NS = Y ~ -1 + intercept3 + #intercept
                  bio1 + bio15 + aPET + Pop #Environmental covariates and population as PhyCoRT Tier 3


NSMod = inla(Frm.NS, 
              data = inla.stack.data(Ind.stk, #Data stack
                                     spde=spde1), 
              family = "binomial", 
              verbose = TRUE,
              control.fixed = list(prec = 0.1, prec.intercept = 0.001), 
              control.predictor = list(
                A = inla.stack.A(Ind.stk), 
                compute = TRUE, 
                link = 1), 
              control.results = list(return.marginals.random = TRUE,
                                     return.marginals.predictor = TRUE),
              control.compute=list(dic = TRUE, cpo = TRUE, waic = TRUE)) 

summary(NSMod)
